/**
 * 
 */
package ca.uqam.info.latece.sboa.impl.amatrix;

import java.util.HashSet;
import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Iterator;

import no.uib.cipr.matrix.Matrix;
import no.uib.cipr.matrix.Vector;
import no.uib.cipr.matrix.DenseMatrix;

import ca.uqam.info.latece.sboa.impl.selection.AssignmentMethodsImpl;

import ca.uqam.info.latece.sboa.inter.ograph.Edge;
import ca.uqam.info.latece.sboa.inter.ograph.Vertice;
import ca.uqam.info.latece.sboa.inter.ograph.VerticesPool;
import ca.uqam.info.latece.sboa.inter.ograph.OntologyGraph;

import ca.uqam.info.latece.sboa.inter.amatrix.AdjacencyMatrix;

import ca.uqam.info.latece.sboa.inter.selection.AssignmentMethods;
import ca.uqam.info.latece.sboa.inter.amatrix.OLAMatrixUpdate;

/**
 * @author J.F. Djoufak
 *
 */
public class OLAMatrixUpdateImpl implements OLAMatrixUpdate {

	int[] verticesIds = null;
	Matrix config = null;
	Matrix update = null;
	
	ArrayList<Integer> lNeigh = null;
	
	VerticesPool leftPool = null;
	VerticesPool rightPool = null;
	
	Hashtable<Integer,Integer> leftCorr = new Hashtable<Integer,Integer>();
	Hashtable<Integer,Integer> rightCorr = new Hashtable<Integer,Integer>();
	
	Hashtable<Integer,Integer> IdToLine = new Hashtable<Integer,Integer>();
	Hashtable<Integer,Integer> IdToColumn = new Hashtable<Integer,Integer>();
	Hashtable<Integer,Integer> IdToLabel = new Hashtable<Integer,Integer>();
	
	ArrayList<Integer> usefulIds = new ArrayList<Integer>();
	
	int[] dynIndices = null;
	
	AssignmentMethods tps = new AssignmentMethodsImpl();
	
	int dynIndSz = 0;
	
	boolean adapt = true;
	boolean select = true;
	

	public OLAMatrixUpdateImpl(int[] verIds, int[] statInd, int mSz,
			Hashtable<Integer,Integer> leftC,Hashtable<Integer,Integer> rightC,
			VerticesPool lPool, VerticesPool rPool) {
		verticesIds = verIds;
		leftCorr = leftC; 
		rightCorr = rightC;
		leftPool = lPool;
		rightPool = rPool;
		int statSz = statInd.length;
		dynIndSz = mSz - statSz;
		dynIndices = new int[dynIndSz];
		HashSet<Integer> statI = new HashSet<Integer>();
		for (int i = 0 ; i < statSz ; i++){
			statI.add(new Integer(statInd[i]));
		}
		int k = 0;
		for (int j = 0 ; j < mSz ; j++){
			if (!statI.contains(new Integer(j))){
				dynIndices[k] = j;
				k++;
			}
		}
	}
	
	public Matrix matrixUpdate(OntologyGraph mGraph, AdjacencyMatrix aMatrix, Vector simVect,
			boolean adpt, boolean sel) {
		adapt = adpt;
		select = sel;
		update = aMatrix.getMatrix();
		int sz = aMatrix.getNumberOfLines();
		for (int i = 0 ; i < dynIndSz ; i++){
			int line = dynIndices[i];
			Hashtable<Integer,Integer> entriesToUpdate = getSelectedEntries(line,mGraph,aMatrix,simVect);
			lineUpdate(line,sz,entriesToUpdate,mGraph,aMatrix);
		}
		return update;
	}

	public Hashtable<Integer,Integer> getSelectedEntries(int line, OntologyGraph mGraph, 
			AdjacencyMatrix aMatrix, Vector simVect) {
		Hashtable<Integer,Integer> result = new Hashtable<Integer,Integer>();
		int nbLab = mGraph.getEdgePool().numberOfPart();
		usefulIds.clear();
		for (int i = 0; i < nbLab ; i++){
			Integer labI = new Integer(i);
			Matrix M = buildLineEntriesSelectionMatrix(line,i,mGraph,aMatrix,simVect);
			if ((select == true) && ((config.numRows() > 1) || (config.numColumns() > 1))){
				Matrix assignment = tps.greedy(M);
				int nRows = assignment.numRows();
				for (int j = 0 ; j < nRows ; j++){
					int lineInd = (int)assignment.get(j,0);
					int colInd = (int)assignment.get(j,1);
					Integer key = new Integer((int)config.get(lineInd,colInd));
					result.put(key,labI);
				}
			}
			else{
				Iterator iter = lNeigh.iterator();
				while(iter.hasNext()){
					Integer key = (Integer)iter.next();
					result.put(key,labI);
				}
			}
		}
		return result;
	}
	
	public Matrix buildLineEntriesSelectionMatrix(int line, int labId, OntologyGraph mGraph, 
			AdjacencyMatrix mMatrix, Vector simVect) {
		int Id = verticesIds[line];
		Vertice ver = (Vertice)mGraph.getVerticePool().get(Id);
		lNeigh = ver.getNeighborhood(labId,false);
		int msz = ver.getNeighborhoodSize(labId,false);
		Matrix result = buildMatrix(lNeigh,msz);
		for (int i = 0 ; i < msz ; i++){
			Integer I = lNeigh.get(i);
			int svId = mMatrix.getLineIndice(I.intValue());
			double score = simVect.get(svId);
			Integer lineId = leftCorr.get(I);
			int rlineId = IdToLine.get(lineId).intValue();
			Integer colId = rightCorr.get(I);
			int rcolId = IdToColumn.get(colId).intValue();	
			result.set(rlineId, rcolId, score);
			config.set(rlineId,rcolId,I.intValue());
			usefulIds.add(I);
		}
		return result;
	}
	
	public Matrix buildMatrix(ArrayList<Integer> lNeigh, int sz){
		HashSet<Integer> lines = new HashSet<Integer>();
		HashSet<Integer> columns = new HashSet<Integer>();
		int lsz = 0;
		int csz = 0;
		for (int i = 0 ; i < sz ; i++){
			Integer I = lNeigh.get(i);
			Integer lId = leftCorr.get(I);
			boolean res = lines.add(lId);
			if (res == true){lsz++;}
			Integer rId = rightCorr.get(I);
			res = columns.add(rId);
			if (res == true) {csz++;}
		}
		config = new DenseMatrix(lsz,csz);
		DenseMatrix result = new DenseMatrix(lsz,csz);
		IdToLine.clear();IdToColumn.clear();
		int i = 0;Iterator iter = lines.iterator();
		while(iter.hasNext()){
			Integer Id = (Integer)iter.next();
			IdToLine.put(Id,new Integer(i));i++;
		}
		i = 0;iter = columns.iterator();
		while(iter.hasNext()){
			Integer Id = (Integer)iter.next();
			IdToColumn.put(Id,new Integer(i));i++;
		}
		return result;
	}
	
	public void lineUpdate(int line, int sz, Hashtable<Integer,Integer> entriesToUpdate, 
			OntologyGraph mGraph, AdjacencyMatrix aMatrix) {
		double[] weight = allEdgesWeight(line,mGraph);
		ArrayList<Integer> keys = new ArrayList<Integer>(entriesToUpdate.keySet());
		int ksz = keys.size();
		usefulIds.removeAll(keys);
		int usz = usefulIds.size();
		for (int i = 0 ; i < usz ; i++){
			Integer I = usefulIds.get(i);
			int Id = (I).intValue();
			int col = aMatrix.getColumnIndice(Id);
			update.set(line,col,0.0);
		}
		for (int j = 0 ; j < ksz ; j++){
			Integer J = keys.get(j);
			int Id = (J).intValue();
			int col = aMatrix.getColumnIndice(Id);
			int labId = entriesToUpdate.get(J).intValue();
			int nVal = getNormalizationValue(labId, verticesIds[line],mGraph);
			double wgt = weight[labId]/nVal;
			update.set(line,col,wgt);
		}
	}
	
	public double[] allEdgesWeight(int line, OntologyGraph mGraph){
		Vertice ver = (Vertice)mGraph.getVerticePool().get(verticesIds[line]);
		int nbEd = mGraph.getEdgePool().numberOfPart();
		int[] partSz = new int[nbEd];
		double[] result = new double[nbEd];
		int divF = 0;
		double sum = 0.0;
		double diff = 0.0;
		for(int i = 0 ; i < nbEd ; i++){
			partSz[i] = ver.getNeighborhoodSize(i,false);
			if (partSz[i] > 0){divF++;}
			result[i] = getEdgeWeight(ver,i,mGraph);
			sum = sum +result[i];
		}
		if (adapt == true){
			if (sum < 1){diff = 1.0 - sum;}
			double incr = 0;if (divF > 0){incr = diff/divF;}
			for (int i = 0 ; i < nbEd ; i++){
				if ((partSz[i] > 0) && (i != 8)){
					result[i] = result[i]+incr;
				}
			}
		}
		return result;
	}
	
	public double getEdgeWeight(Vertice ver, int labId, OntologyGraph mGraph){
		double result = 0.0;
		ArrayList<Integer> edgIds = ver.getEdges(labId,false);
		if (!edgIds.isEmpty()){
			int eId = edgIds.get(0);
			result = ((Edge)mGraph.getEdgePool().get(eId)).getWeight();
		}
		return result;
	}
	
	public int getNormalizationValue(int labId, int mgId, OntologyGraph mGraph){
		int result = 0;
		Integer mgI = new Integer(mgId);
		int lId = leftCorr.get(mgI).intValue();
		int rId = rightCorr.get(mgI).intValue();
		Vertice lver = (Vertice)leftPool.get(lId);
		Vertice rver = (Vertice)rightPool.get(rId);
		int lpSz = lver.getNeighborhoodSize(labId,false);
		int rpSz = rver.getNeighborhoodSize(labId,false);
		if (adapt == true){
			result = Math.max(lpSz,rpSz);
		}
		else{
			result = lpSz*rpSz;
		}
		return result;
	}
	
	public void clearAll(){
		verticesIds = null;
		leftCorr.clear();
		leftCorr = null;
		rightCorr.clear();
		rightCorr = null;
		IdToLine.clear();
		IdToLine = null;
		IdToColumn.clear();
		IdToColumn = null;
		lNeigh.clear();
		lNeigh = null;
		config = null;
	}

}
