/**
 * 
 */
package ca.uqam.info.latece.sboa.impl.encodings;

import java.net.URI;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.semanticweb.owl.align.Parameters;
import org.semanticweb.owl.impl.model.OWLClassImpl;
import org.semanticweb.owl.impl.model.OWLConcreteDataImpl;
import org.semanticweb.owl.impl.model.OWLConcreteDataTypeImpl;
import org.semanticweb.owl.impl.model.OWLDataAllRestrictionImpl;
import org.semanticweb.owl.impl.model.OWLDataCardinalityRestrictionImpl;
import org.semanticweb.owl.impl.model.OWLDataPropertyImpl;
import org.semanticweb.owl.impl.model.OWLIndividualImpl;
import org.semanticweb.owl.impl.model.OWLObjectAllRestrictionImpl;
import org.semanticweb.owl.impl.model.OWLObjectCardinalityRestrictionImpl;
import org.semanticweb.owl.impl.model.OWLObjectPropertyImpl;
import org.semanticweb.owl.impl.model.OWLSubPropertyAxiomImpl;

import org.semanticweb.owl.model.OWLEntity;
import org.semanticweb.owl.model.OWLOntology;
//import org.semanticweb.owl.model.OWLClass;
import org.semanticweb.owl.model.OWLProperty;
import org.semanticweb.owl.model.OWLException;
import org.semanticweb.owl.model.OWLAnnotationInstance;
import org.semanticweb.owl.model.OWLObjectProperty;

import ca.uqam.info.latece.sboa.impl.ograph.EdgeImpl;
import ca.uqam.info.latece.sboa.impl.ograph.OntologyGraphImpl;
import ca.uqam.info.latece.sboa.impl.ograph.VerticeImpl;
import ca.uqam.info.latece.sboa.impl.utils.LabelTokenizer;
import ca.uqam.info.latece.sboa.impl.utils.OLATools;
import ca.uqam.info.latece.sboa.impl.weights.OLAWeightCapture;
import ca.uqam.info.latece.sboa.inter.encodings.OLAOntologyGraph;
import ca.uqam.info.latece.sboa.inter.ograph.Edge;
import ca.uqam.info.latece.sboa.inter.ograph.OntologyGraph;
import ca.uqam.info.latece.sboa.inter.ograph.Vertice;

/**
 * @author J.F. Djoufak
 *
 */
public class OLAOntologyGraphImpl implements OLAOntologyGraph {
	
	private Object[] cSet;
	private Object[] dSet;
	private Object[] iSet;
	private Object[] dpSet;
	private Object[] opSet;
	
	private int csz = 0;
	private int dsz = 0;
	private int isz = 0;
	private int dpsz = 0;
	private int opsz = 0;
	
	private Hashtable<String,Integer> classToId = new Hashtable<String,Integer>();
	private Hashtable<String,Integer> datatypeToId = new Hashtable<String,Integer>();
	private Hashtable<String,Integer> relationToId = new Hashtable<String,Integer>();
	private Hashtable<String,Integer> propertyToId = new Hashtable<String,Integer>();
	private Hashtable<String,Integer> objectToId = new Hashtable<String,Integer>();
	private Hashtable<String,Integer> propertyInstanceToId = new Hashtable<String,Integer>();
	private Hashtable<String,Integer> dataValueToId = new Hashtable<String,Integer>();
	private Hashtable<String,String> nullVerticeToName = new Hashtable<String,String>();
	private Hashtable<String,Integer> tokenToId = new Hashtable<String,Integer>();
	private Hashtable<String,Integer> cardinalityToId = new Hashtable<String,Integer>();
	
	private int currentVerticeId = 0;
	private int currentEdgeId = 0; 
	
	private OLATools ot = null;
	private OLAWeightCapture owg = null;
	
	int NAMEID = 9;
	
	public OLAOntologyGraphImpl(Parameters param) {
		owg = new OLAWeightCapture(param);
		ot = new OLATools(param);
	}

	public OntologyGraph build(OWLOntology onto) {
		initGraphConstruction();
		String[] catSet = extractCategories(onto);
		String[] labSet = extractLabels(onto);
		//System.out.println("first");
		Vertice[] vSet = extractVertices(onto);
		//System.out.println("second");
		Edge[] eSet = extractEdges(onto,vSet);
		//System.out.println("third");
		updatePropertyInstanceName(vSet);
		OntologyGraph result = new OntologyGraphImpl(catSet,labSet);
		result.StraightforwardBuild(vSet,eSet);
		clearAll();
		return result;
	}
	
	public void initGraphConstruction(){
		currentVerticeId = 0;
		currentEdgeId = 0;
		csz = 0;
		dsz = 0;
		isz = 0;
		dpsz = 0;
		opsz = 0;
		classToId.clear();
		datatypeToId.clear();
		relationToId.clear();
		propertyToId.clear();
		objectToId.clear();
		propertyInstanceToId.clear();
		dataValueToId.clear();
		nullVerticeToName.clear();
		tokenToId.clear();
		cardinalityToId.clear();
	}

	public String[] extractCategories(OWLOntology onto) {
		String[] result = {"Class", "Object", "Relation", "Property", 
				"PropertyInstance", "DataType", "DataValue", "Token", "Cardinality"};
		return result;
	}

	public String[] extractLabels(OWLOntology onto) {
		String[] result = {"subsumption", "instantiation", "attribution", "domain", "range",
				"restriction", "valuation", "all", "card", "name"};
		return result;
	}
	
	public Vertice[] extractVertices(OWLOntology onto) {
		ArrayList<Vertice> vSet = new ArrayList<Vertice>();
		buildEntitiesSets(onto);
		vSet.addAll(buildVerticesFromClasses(onto));
		vSet.addAll(buildVerticesFromDatatypes());
		vSet.addAll(buildVerticesFromObjectProperties(onto));
		vSet.addAll(buildVerticesFromDataProperties(onto));
		vSet.addAll(buildVerticesFromIndividuals(onto));
		Vertice[] result = new Vertice[currentVerticeId];
		for (int i = 0; i < currentVerticeId ; i++){
			result[i] = vSet.get(i);
		}
		return result;
	}
	
	public Edge[] extractEdges(OWLOntology onto, Vertice[] v) {
		ArrayList<Edge> eSet = new ArrayList<Edge>();
		eSet.addAll(encodeSubsumptionEdgesBetweenClasses(onto));
		eSet.addAll(encodeSubsumptionEdgesBetweenProperties(onto));
		eSet.addAll(encodeDomainEdgesBetweenDataPropertiessAndClasses(onto));
		eSet.addAll(encodeDomainEdgesBetweenObjectPropertiessAndClasses(onto));
		eSet.addAll(encodeRangeEdgesBetweenDataPropertiessAndDatatype(onto));
		eSet.addAll(encodeRangeEdgesBetweenObjectPropertiesAndClasses(onto));
		eSet.addAll(encodeRestrictionEdgeBetweenPropertyAndCardinality(v));
		eSet.addAll(encodeInstantiationEdgesBetweenClassesAndObjects(onto));
		eSet.addAll(encodeEdgesBetweenClassesAndProperties(onto));
		Edge[] result = new Edge[currentEdgeId];
		for (int i = 0; i < currentEdgeId ; i++){
			result[i] = eSet.get(i);
		}
		return result;
	}
	
	public void buildEntitiesSets(OWLOntology onto){
		try{
			cSet = onto.getClasses().toArray();
			dSet = onto.getDatatypes().toArray();
			iSet = onto.getIndividuals().toArray();
			dpSet = onto.getDataProperties().toArray();
			opSet = onto.getObjectProperties().toArray();	
			csz = cSet.length;
			dsz = dSet.length;
			isz = iSet.length;
			dpsz = dpSet.length;
			opsz = opSet.length;
		}
		catch(OWLException ex){
			ex.printStackTrace();
		}
	}
	
	public void addVertice(ArrayList<Vertice> result, Hashtable<String, Integer> ht, 
			String cat, String lab, String comm, String ann, OWLEntity e1, OWLEntity e2, String key){
		if ((ot.stringFilter(lab)==false) && (!ht.containsKey(lab) && (!ht.containsKey(key)))){
			Vertice v = new VerticeImpl(currentVerticeId,cat,lab,comm,ann,0.0,e1,e2);
			if (!key.equalsIgnoreCase("")){
				ht.put(key, currentVerticeId);
			}
			else{
				ht.put(lab, currentVerticeId);
			}
			result.add(v);
			currentVerticeId++;
		}
		
	}
	
	private String extractAnnotation(OWLEntity ent, OWLOntology onto){
		String result = "";
		try{
			Set ann = ent.getAnnotations(onto);
			Iterator iter = ann.iterator();
			while(iter.hasNext()){
				OWLAnnotationInstance annI = (OWLAnnotationInstance)iter.next();
				OWLConcreteDataImpl cdi = (OWLConcreteDataImpl)annI.getContent();
				result = result + " " + cdi.getValue().toString();
			}
		}
		catch(Exception e){
			
		}
		return result;
	}
	
	public ArrayList<Vertice> buildVerticesFromClasses(OWLOntology onto){
		ArrayList<Vertice> result = new ArrayList<Vertice>();
		for(int i = 0 ; i < csz ; i++){
			OWLClassImpl owlC = (OWLClassImpl)cSet[i];
			boolean auth = owlC != null;
			if(auth == true){
				URI curi = owlC.getURI();
				String cName = ot.getName(curi,"Class",1);
				String ann = extractAnnotation(owlC,onto);
				addVertice(result,classToId,"Class",cName,"",ann,owlC,null,curi.toString());
				result.addAll(buildPropertyVerticesFromClass(owlC,onto));
			}
		}
		addVertice(result,classToId,"Class","Null","","",null,null,"");
		return result;
	}
	
	public ArrayList<Vertice> buildPropertyVerticesFromClass(
			OWLClassImpl owlC, OWLOntology onto){
		ArrayList<Vertice> result = new ArrayList<Vertice>();
		Object[] sup = owlC.getSuperClasses().toArray();
		int supsz = sup.length;
		for (int i = 0 ; i < supsz ; i++){
			Object roc = sup[i];
			if (roc instanceof OWLObjectCardinalityRestrictionImpl){
				result.addAll(buildVerticesFromObjectCardinalityRestriction(
						(OWLObjectCardinalityRestrictionImpl)roc,onto));
			}
			else{
				if (roc instanceof OWLDataCardinalityRestrictionImpl){
					result.addAll(buildVerticesFromDataCardinalityRestriction(
							(OWLDataCardinalityRestrictionImpl)roc,onto));
				}
				else{
					if (roc instanceof OWLObjectAllRestrictionImpl){
						result.addAll(buildVerticesFromObjectAllRestriction(
								(OWLObjectAllRestrictionImpl)roc,onto));	
					}
					else{
						if (roc instanceof OWLDataAllRestrictionImpl){
							result.addAll(buildVerticesFromDataAllRestriction(
									(OWLDataAllRestrictionImpl)roc,onto));
						}
					}
				}
			}
		}
		return result;
	}
	
	public ArrayList<Vertice>buildVerticesFromObjectCardinalityRestriction(
			OWLObjectCardinalityRestrictionImpl ocr, OWLOntology onto){
		ArrayList<Vertice> result = new ArrayList<Vertice>();
		try{
			OWLObjectProperty op = (OWLObjectPropertyImpl)ocr.getObjectProperty();
			boolean auth = op != null;
			if (auth == true){
				URI opuri= op.getURI();
				String opName = ot.getName(opuri,"Property",1);
				int least = ocr.getAtLeast();
				int most = ocr.getAtMost();
				String card = ot.generateCardinalityString(least, most);
				String ann = extractAnnotation(op,onto);
				addVertice(result,propertyToId,"Property",opName,card,ann,op,null,opuri.toString());
				result.addAll(buildCardinalityVertice(card));
			}
		}
		catch(OWLException ex){
			ex.printStackTrace();
		}
		return result;
	}
	
	public ArrayList<Vertice>buildVerticesFromDataCardinalityRestriction(
			OWLDataCardinalityRestrictionImpl dcr, OWLOntology onto){
		ArrayList<Vertice> result = new ArrayList<Vertice>();
		try{
			OWLDataPropertyImpl dp = (OWLDataPropertyImpl)dcr.getDataProperty();
			boolean auth = dp != null;
			if (auth == true){
				URI dpuri= dp.getURI();
				String dpName = ot.getName(dpuri,"Property",1);
				int least = dcr.getAtLeast();
				int most = dcr.getAtMost();
				String card = ot.generateCardinalityString(least, most);
				String ann = extractAnnotation(dp,onto);
				addVertice(result,propertyToId,"Property",dpName,card,ann,dp,null,dpuri.toString());
				result.addAll(buildCardinalityVertice(card));
			}
		}
		catch(OWLException ex){
			ex.printStackTrace();
		}
		return result;
	}
	
	public ArrayList<Vertice>buildVerticesFromObjectAllRestriction(
			OWLObjectAllRestrictionImpl oar, OWLOntology onto){
		ArrayList<Vertice> result = new ArrayList<Vertice>();
		try{
			OWLObjectPropertyImpl op = (OWLObjectPropertyImpl)oar.getObjectProperty();
			boolean auth = op != null;
			if (auth == true){
				URI opuri= op.getURI();
				String opName = ot.getName(opuri,"Property",1);
				String card = "all";
				String ann = extractAnnotation(op,onto);
				addVertice(result,propertyToId,"Property",opName,card,ann,op,null,opuri.toString());
				result.addAll(buildCardinalityVertice(card));
			}
		}
		catch(Exception ex){
			ex.printStackTrace();
		}
		return result;
	}
	
	public ArrayList<Vertice>buildVerticesFromDataAllRestriction(
			OWLDataAllRestrictionImpl dar, OWLOntology onto){
		ArrayList<Vertice> result = new ArrayList<Vertice>();
		try{
			OWLDataPropertyImpl dp = (OWLDataPropertyImpl)dar.getDataProperty();
			boolean auth = dp != null;
			if (auth == true){
				URI dpuri= dp.getURI();
				String dpName = ot.getName(dpuri,"Property",1);
				String card = "all";
				String ann = extractAnnotation(dp,onto);
				addVertice(result,propertyToId,"Property",dpName,card,ann,dp,null,dpuri.toString());
				result.addAll(buildCardinalityVertice(card));
			}
		}
		catch(Exception ex){
			ex.printStackTrace();
		}
		return result;
	}
	
	public ArrayList<Vertice> buildCardinalityVertice(String token){
		ArrayList<Vertice> result = new ArrayList<Vertice>();
		if (!cardinalityToId.containsKey(token)) {
			addVertice(result,cardinalityToId,"Cardinality",token,"","",null,null,"");
		}
		return result;
	}
	
	public ArrayList<Vertice> buildVerticesFromDatatypes(){
		ArrayList<Vertice> result = new ArrayList<Vertice>();
		for(int i = 0 ; i < dsz ; i++){
			OWLConcreteDataTypeImpl owlD = (OWLConcreteDataTypeImpl)dSet[i];
			boolean auth = owlD != null;
			if (auth == true){
				URI duri = owlD.getURI();
				String dName = ot.getName(duri,"DataType",1);
				addVertice(result,datatypeToId,"DataType",dName,"","",null,null,"");
			}
		}
		addVertice(result,datatypeToId,"DataType","null","","",null,null,"");
		return result;
	}
	
	public ArrayList<Vertice> buildVerticesFromDataProperties(OWLOntology onto){
		ArrayList<Vertice> result = new ArrayList<Vertice>();
		for(int i = 0 ; i < dpsz ; i++){
			OWLDataPropertyImpl owlDP = (OWLDataPropertyImpl)dpSet[i];
			boolean auth = owlDP != null;
			if (auth == true){
				URI dpuri = owlDP.getURI();
				String dpName = ot.getName(dpuri,"Relation",1);
				String ann = extractAnnotation(owlDP,onto);
				addVertice(result,relationToId,"Relation",dpName,"",ann,owlDP,null,dpuri.toString());
				result.addAll(buildVerticesFromDataPropertyRange(owlDP,onto));
			}
		}
		return result;	
	}
	
	public ArrayList<Vertice> buildVerticesFromDataPropertyRange(OWLDataPropertyImpl opi, 
			OWLOntology onto){
		ArrayList<Vertice> result = new ArrayList<Vertice>();
		Set rg = opi.getRanges();
		Iterator iter = rg.iterator();
		while (iter.hasNext()){
			Object obj = iter.next();
			if (obj instanceof OWLConcreteDataTypeImpl){
				OWLConcreteDataTypeImpl cdi = (OWLConcreteDataTypeImpl)obj;
				boolean auth = cdi != null;
				if (auth == true){
					URI duri = cdi.getURI();
					String dName = ot.getName(duri, "DataType", 0);
					addVertice(result,datatypeToId,"DataType",dName,"","",null,null,"");
				}
			}
		}
		return result;
	}
	
	public ArrayList<Vertice> buildVerticesFromObjectProperties(OWLOntology onto){
		ArrayList<Vertice> result = new ArrayList<Vertice>();
		for(int i = 0 ; i < opsz ; i++){
			OWLObjectPropertyImpl owlOP = (OWLObjectPropertyImpl)opSet[i];
			boolean auth = owlOP != null;
			if (auth == true){
				URI opuri = owlOP.getURI();
				String opName = ot.getName(opuri,"Relation",1);
				String ann = extractAnnotation(owlOP,onto);
				addVertice(result,relationToId,"Relation",opName,"",ann,owlOP,null,opuri.toString());
			}
		}
		return result;
	}
	
	public ArrayList<Vertice> buildVerticesFromIndividuals(OWLOntology onto){
		ArrayList<Vertice> result = new ArrayList<Vertice>();
		for(int i = 0 ; i < isz ; i++){
			OWLIndividualImpl owlI = (OWLIndividualImpl)iSet[i];
			boolean auth = owlI != null;
			if (auth == true){
				URI iuri = owlI.getURI();
				String type = ot.getType(owlI,onto,1); 
				String iName = ot.getName(iuri,type,1);
				String ann = extractAnnotation(owlI,onto);
				addVertice(result,objectToId,"Object",iName,"",ann,owlI,null,"");
				String idS = ot.getIdentification(owlI,onto);
				if((!idS.equalsIgnoreCase(""))&&(!nullVerticeToName.containsKey(idS))){
					nullVerticeToName.put(idS,iName);
				}	
			}
		}
		for(int j = 0 ; j < isz ; j++){
			OWLIndividualImpl owlI = (OWLIndividualImpl)iSet[j];
			boolean auth = owlI != null;
			if (auth == true){
				result.addAll(buildVerticesFromDataPropertyInstances(owlI,onto));
				result.addAll(buildVerticesFromObjectPropertyInstances(owlI,onto));
			}
		}
		return result;
	}
	
	public ArrayList<Vertice> buildVerticesFromDataPropertyInstances(OWLIndividualImpl ind, 
			OWLOntology onto){
		ArrayList<Vertice> result = new ArrayList<Vertice>();
		Map dpmap = ind.getDataPropertyValues(onto);
		Object[] di = dpmap.keySet().toArray();
		int sz = di.length;
		for(int i=0;i<sz;i++){
			OWLDataPropertyImpl dpi = (OWLDataPropertyImpl)di[i];
			boolean auth1 = dpi != null;
			if (auth1 == true){
				HashSet val = (HashSet)dpmap.get(dpi);
				Iterator iter = val.iterator();
				while (iter.hasNext()){
					OWLConcreteDataImpl cdi = (OWLConcreteDataImpl)iter.next();
					boolean auth2 = cdi != null;
					if (auth2 == true){
						String piName = cdi.getValue().toString();
						addVertice(result,propertyInstanceToId,"PropertyInstance",piName,"","",dpi,null,"");
						addVertice(result,dataValueToId,"DataValue",piName,"","",null,null,"");
					}
				}
			}
		}
		return result;
	}
	
	public ArrayList<Vertice> buildVerticesFromObjectPropertyInstances(
			OWLIndividualImpl ind, OWLOntology onto){
		ArrayList<Vertice> result = new ArrayList<Vertice>();
		Map opmap = ind.getObjectPropertyValues(onto);
		Object[] dj = opmap.keySet().toArray();
		for(int i=0;i<dj.length;i++){
			OWLObjectPropertyImpl opi = (OWLObjectPropertyImpl)dj[i];
			HashSet val = (HashSet)opmap.get(opi);
			Iterator iter = val.iterator();
			while (iter.hasNext()){
				OWLIndividualImpl cdi = (OWLIndividualImpl)iter.next();
				boolean auth = cdi != null;
				if (auth == true){
					String piName =  null;
					URI cdiuri = cdi.getURI(); 
					if (cdiuri == null){
						piName = nullVerticeToName.get(ot.getIdentification(cdi,onto));
					}
					else{
						piName = ot.getName(cdiuri, "", 0);
					}
					addVertice(result,propertyInstanceToId,"PropertyInstance",piName,"","",cdi,null,"");
				}
			}
		}
		return result;
	}
	
	public void addEdge(ArrayList<Edge> result, String lab, int hd, int tl, double wgt){
		Edge e = new EdgeImpl(currentEdgeId,lab,wgt,hd,tl);
		result.add(e);
		currentEdgeId++;
	}
	
	public ArrayList<Edge> encodeSubsumptionEdgesBetweenClasses(OWLOntology onto){
		ArrayList<Edge> result = new ArrayList<Edge>();
		for(int i = 0 ; i < csz ; i++){
			OWLClassImpl owlC = (OWLClassImpl)cSet[i];
			boolean auth1 = owlC != null;
			if (auth1 == true){
				URI curi = owlC.getURI();
				int cid = classToId.get(curi.toString()).intValue();
				try{
					Object[] subC = owlC.getSubClasses(onto).toArray();
					int ssz = subC.length;
					for (int j = 0 ; j < ssz ; j++){
						OWLClassImpl owlsC = (OWLClassImpl)subC[j];
						boolean auth2 = owlsC != null;
						if (auth2 == true){
							URI scuri = owlsC.getURI();
							int scid = classToId.get(scuri.toString()).intValue();
							double wgt = owg.getWeight(4);
							addEdge(result,"subsumption",cid,scid,wgt);
						}
					}
				}
				catch(OWLException ex){
					ex.printStackTrace();
				}
			}
		}
		return result;
	}
	
	public ArrayList<Edge> encodeSubsumptionEdgesBetweenProperties(OWLOntology onto){
		ArrayList<Edge> result = new ArrayList<Edge>();
			try{
				Object[] PA = onto.getPropertyAxioms().toArray();
				int ssz = PA.length;
				for (int j = 0 ; j < ssz ; j++){
					OWLSubPropertyAxiomImpl spa = (OWLSubPropertyAxiomImpl)PA[j];
					OWLProperty sup = spa.getSuperProperty();
					OWLProperty sub = spa.getSubProperty();
					boolean auth1 = sup != null;
					boolean auth2 = sub != null;
					if ( (auth1 == true) && (auth2 == true)){
						URI supUri = sup.getURI();
						URI subUri  = sub.getURI();
						int supId = relationToId.get(supUri.toString());
						int subId = relationToId.get(subUri.toString());
						double wgt = owg.getWeight(20);//24
						addEdge(result,"subsumption",supId,subId,wgt);
					}
				}
			}
			catch(OWLException ex){
				ex.printStackTrace();
			}
		return result;
	}
	
	public ArrayList<Edge> encodeDomainEdgesBetweenDataPropertiessAndClasses(OWLOntology onto){
		ArrayList<Edge> result = new ArrayList<Edge>();
		for(int i = 0 ; i < dpsz ; i++){
			OWLDataPropertyImpl owlDP = (OWLDataPropertyImpl)dpSet[i];
			boolean auth1 = owlDP != null;
			if (auth1 == true){
				URI dpuri = owlDP.getURI();
				int dpid = relationToId.get(dpuri.toString()).intValue();
				Object[] dpDom = owlDP.getDomains(onto).toArray();
				int domsz = dpDom.length;
				if (domsz == 0){
					int ncid = classToId.get("Null").intValue();
					double wgt = 0.0;//owg.getWeight(18);
					addEdge(result,"domain",ncid,dpid,wgt);
				}
				for (int j = 0 ; j < domsz ; j++){
					Object[] operand = ot.getOperand(dpDom[j]);
					int opsz = operand.length;
					for (int k = 0 ; k < opsz ; k++){
						OWLClassImpl owlC = (OWLClassImpl)operand[k];
						boolean auth2 = owlC != null;
						if (auth2 == true){
							URI curi = owlC.getURI();
							int cid = classToId.get(curi.toString()).intValue();
							double wgt = owg.getWeight(18);
							addEdge(result,"domain",cid,dpid,wgt);
							result.addAll(encodeAttributionEdgesBetweenClassesAndProperties(curi.toString(),dpuri.toString()));
							result.addAll(encodeSubsumptionEdgesBetweenPropertiesAndRelations(dpuri.toString(),dpuri.toString()));
						}
					}
				}
			}
		}
		return result;
	}
	
	public ArrayList<Edge> encodeDomainEdgesBetweenObjectPropertiessAndClasses(OWLOntology onto){
		ArrayList<Edge> result = new ArrayList<Edge>();
		for(int i = 0 ; i < opsz ; i++){
			OWLObjectPropertyImpl owlOP = (OWLObjectPropertyImpl)opSet[i];
			boolean auth1 = owlOP != null;
			if (auth1 == true){
				URI opuri = owlOP.getURI();
				int opid = relationToId.get(opuri.toString()).intValue();
				Object[] opDom = owlOP.getDomains(onto).toArray();
				int domsz = opDom.length;
				if (domsz == 0){
					int ncid = classToId.get("Null").intValue();
					double wgt = 0.0;//owg.getWeight(22);
					addEdge(result,"domain",ncid,opid,wgt);
				}
				for (int j = 0 ; j < domsz ; j++){
					Object[] operand = ot.getOperand(opDom[j]);
					int opsz = operand.length;
					for (int k = 0 ; k < opsz ; k++){
						OWLClassImpl owlC = (OWLClassImpl)operand[k];
						boolean auth2 = owlC != null;
						if (auth2 == true){
							URI curi = owlC.getURI();
							int cid = classToId.get(curi.toString()).intValue();
							double wgt = owg.getWeight(22);
							addEdge(result,"domain",cid,opid,wgt);
							result.addAll(encodeAttributionEdgesBetweenClassesAndProperties(curi.toString(),opuri.toString()));
							result.addAll(encodeSubsumptionEdgesBetweenPropertiesAndRelations(opuri.toString(),opuri.toString()));
						}
					}
				}
			}
		}
		return result;
	}
	
	public ArrayList<Edge> encodeAttributionEdgesBetweenClassesAndProperties(
			String cName, String pName){
		ArrayList<Edge> result = new ArrayList<Edge>();
		if (propertyToId.containsKey(pName)){
			int cid = classToId.get(cName).intValue();
			int pid = propertyToId.get(pName).intValue();
			double wgt = owg.getWeight(5);//6
			addEdge(result,"attribution",pid,cid,wgt);
		}
		return result;
	}
	
	public ArrayList<Edge> encodeSubsumptionEdgesBetweenPropertiesAndRelations(
			String pName, String rName){
		ArrayList<Edge> result = new ArrayList<Edge>();
		if (propertyToId.containsKey(pName)){
			int rid = relationToId.get(rName).intValue();
			int pid = propertyToId.get(pName).intValue();
			double wgt = owg.getWeight(11);//16
			addEdge(result,"subsumption",rid,pid,wgt);
		}
		return result;
	}
	
	public ArrayList<Edge> encodeRangeEdgesBetweenDataPropertiessAndDatatype(OWLOntology onto){
		ArrayList<Edge> result = new ArrayList<Edge>();
		for(int i = 0 ; i < dpsz ; i++){
			OWLDataPropertyImpl owlDP = (OWLDataPropertyImpl)dpSet[i];
			boolean auth1 = owlDP != null;
			if (auth1 == true){
				URI dpuri = owlDP.getURI();
				int dpid = relationToId.get(dpuri.toString()).intValue();
				Object[] dpRan = owlDP.getRanges().toArray();
				int ransz = dpRan.length;
				if (ransz == 0){
					int ndid = datatypeToId.get("null").intValue();
					double wgt = 0.0;//owg.getWeight(19);
					addEdge(result,"range",ndid,dpid,wgt);
				}
				for (int j = 0 ; j < ransz ; j++){
					Object[] operand = ot.getOperand(dpRan[j]);
					int opsz = operand.length;
					for (int k = 0 ; k < opsz ; k++){
						Object obj = operand[k];
						if (obj instanceof OWLConcreteDataTypeImpl){
							OWLConcreteDataTypeImpl owlD = (OWLConcreteDataTypeImpl)obj;
							boolean auth2 = owlD != null;
							if (auth2 == true){
								URI duri = owlD.getURI();
								String dName = ot.getName(duri,"DataType",1);
								int did = datatypeToId.get(dName).intValue();
								double wgt = owg.getWeight(19);
								addEdge(result,"range",did,dpid,wgt);
								result.addAll(encodeAllEdgesBetweenPropertiesAndDatatypes(dpuri.toString(),dName));
							}
						}
					}
				}
			}
		}
		return result;
	}
	
	public ArrayList<Edge> encodeAllEdgesBetweenPropertiesAndDatatypes(
			String pName, String dName){
		ArrayList<Edge> result = new ArrayList<Edge>();
		if (propertyToId.containsKey(pName)){
			int did = datatypeToId.get(dName).intValue();
			int pid = propertyToId.get(pName).intValue();
			double wgt = owg.getWeight(13);
			addEdge(result,"all",did,pid,wgt);
		}
		return result;
	}
	
	public ArrayList<Edge> encodeRangeEdgesBetweenObjectPropertiesAndClasses(OWLOntology onto){
		ArrayList<Edge> result = new ArrayList<Edge>();
		for(int i = 0 ; i < opsz ; i++){
			OWLObjectPropertyImpl owlOP = (OWLObjectPropertyImpl)opSet[i];
			boolean auth1 = owlOP != null;
			if (auth1 == true){
				URI opuri = owlOP.getURI();
				int opid = relationToId.get(opuri.toString()).intValue();
				Object[] opRan = owlOP.getRanges().toArray();
				int ransz = opRan.length;
				if (ransz == 0){
					int ncid = classToId.get("Null").intValue();
					double wgt = 0.0;//owg.getWeight(23);
					addEdge(result,"range",ncid,opid,wgt);
				}
				for (int j = 0 ; j < ransz ; j++){
					Object[] operand = ot.getOperand(opRan[j]);
					int opsz = operand.length;
					for (int k = 0 ; k < opsz ; k++){
						OWLClassImpl owlC = (OWLClassImpl)operand[k];
						boolean auth2 = owlC != null;
						if (auth2== true){
							URI curi = owlC.getURI();
							int cid = classToId.get(curi.toString()).intValue();
							double wgt = owg.getWeight(23);
							addEdge(result,"range",cid,opid,wgt);
							result.addAll(encodeAllEdgesBetweenPropertiesAndClasses(opuri.toString(),curi.toString()));
						}
					}
				}
			}
		}
		return result;
	}
	
	public ArrayList<Edge> encodeAllEdgesBetweenPropertiesAndClasses(
			String pName, String cName){
		ArrayList<Edge> result = new ArrayList<Edge>();
		if (propertyToId.containsKey(pName)){
			int cid = classToId.get(cName).intValue();
			int pid = propertyToId.get(pName).intValue();
			double wgt = owg.getWeight(8);
			addEdge(result,"all",cid,pid,wgt);
		}
		return result;
	}
	
	public ArrayList<Edge> encodeInstantiationEdgesBetweenClassesAndObjects(OWLOntology onto){
		ArrayList<Edge> result = new ArrayList<Edge>();
		try{
		for(int i = 0 ; i < isz ; i++){
			OWLIndividualImpl owlI = (OWLIndividualImpl)iSet[i];
			boolean auth1 = owlI != null;
			if (auth1 == true){
				URI iuri = owlI.getURI();
				String cName = ot.getType(owlI,onto,0);
				if (!cName.equalsIgnoreCase("")){
					String oName = null;
					String idS = ot.getIdentification(owlI,onto);
					String type = ot.getType(owlI,onto,1);
					if(!idS.equalsIgnoreCase("")){
						oName = (String)nullVerticeToName.get(idS);
					}
					else{
						oName = ot.getName(iuri,type,1);
					}
					int cid = classToId.get(cName).intValue();
					int oid = objectToId.get(oName).intValue();
					double wgt = owg.getWeight(3);
					addEdge(result,"instantiation",oid,cid,wgt);
					result.addAll(encodeEdgesBetweenObjectsPropertyInstanceDataPropertiesAndDataTypes(owlI,oName,oid,onto));
					result.addAll(encodeEdgesBetweenObjectsPropertyInstancesAndObjectProperties(owlI,oName,oid,onto));
				}
			}
		}
		}
		catch(Exception ex){
			
		}
		return result;
	}
	
	public ArrayList<Edge> encodeEdgesBetweenObjectsPropertyInstanceDataPropertiesAndDataTypes(
			OWLIndividualImpl ind, String iName, int iId, OWLOntology onto){
		ArrayList<Edge> result = new ArrayList<Edge>();
		Map dpmap = ind.getDataPropertyValues(onto);
		Object[] di = dpmap.keySet().toArray();
		for(int i=0;i<di.length;i++){
			OWLDataPropertyImpl dpi = (OWLDataPropertyImpl)di[i];
			boolean auth1 = dpi != null;
			if (auth1 == true){
				URI dpuri = dpi.getURI();
				//String dtName = ot.getDataPropertyRange(dpi,onto);
				int dpid = relationToId.get(dpuri.toString()).intValue();
				//int dtid = datatypeToId.get(dtName).intValue();
				HashSet val = (HashSet)dpmap.get(dpi);
				Iterator iter = val.iterator();
				while (iter.hasNext()){
					OWLConcreteDataImpl cdi = (OWLConcreteDataImpl)iter.next();
					boolean auth2 = cdi != null;
					if (auth2 == true){
						String piName = cdi.getValue().toString();
						int piid = propertyInstanceToId.get(piName).intValue();
						int dvid = dataValueToId.get(piName).intValue();
						double wgt = owg.getWeight(33);//by default
						addEdge(result,"attribution",iId,piid,wgt);
						wgt = owg.getWeight(26);
						addEdge(result,"attribution",piid,iId,wgt);
						wgt = owg.getWeight(31);
						addEdge(result,"restriction",dpid,piid,wgt);
						wgt = owg.getWeight(30);
						addEdge(result,"valuation",dvid,piid,wgt);
						//wgt = 0.0;
						//addEdge(result,"","","instantiation",null,dtid,dvid,wgt);
					}
				}
			}
		}
		return result;
	}
	
	public ArrayList<Edge> encodeEdgesBetweenObjectsPropertyInstancesAndObjectProperties(
			OWLIndividualImpl ind, String iName, int iId, OWLOntology onto){
		ArrayList<Edge> result = new ArrayList<Edge>();
		Map dpmap = ind.getObjectPropertyValues(onto);
		Object[] di = dpmap.keySet().toArray();
		for(int i=0;i<di.length;i++){
			OWLObjectPropertyImpl opi = (OWLObjectPropertyImpl)di[i];
			boolean auth1 = opi != null;
			if (auth1 == true){
				URI opuri = opi.getURI();
				int opid = relationToId.get(opuri.toString());
				HashSet val = (HashSet)dpmap.get(opi);
				Iterator iter = val.iterator();
				while (iter.hasNext()){
					OWLIndividualImpl oi = (OWLIndividualImpl)iter.next();
					boolean auth2 = oi != null;
					if (auth2 == true){
						URI oiuri = oi.getURI();
						String oName = null;
						String oiIden = ot.getIdentification(oi,onto);
						if(!oiIden.equalsIgnoreCase("")){
							oName = (String)nullVerticeToName.get(oiIden);
						}
						else{
							oName = ot.getName(oiuri,"",0);
						}
						int piid = propertyInstanceToId.get(oName).intValue();
						int oid = objectToId.get(oName);
						double wgt = owg.getWeight(33);
						addEdge(result,"attribution",iId,piid,wgt);
						wgt = owg.getWeight(27);
						addEdge(result,"attribution",piid,iId,wgt);
						wgt = owg.getWeight(34);
						addEdge(result,"restriction",opid,piid,wgt);
						wgt = owg.getWeight(33);//by default
						addEdge(result,"valuation",oid,piid,wgt);
					}
				}
			}
		}
		return result;
	}
	
	public ArrayList<Edge> encodeRestrictionEdgeBetweenPropertyAndCardinality(Vertice[] ver){
		ArrayList<Edge> result = new ArrayList<Edge>();
		Object[] propSet = propertyToId.values().toArray();
		int sz = propSet.length;
		for(int i = 0 ; i < sz ; i++){
			int id = ((Integer)propSet[i]).intValue();
			Vertice prop = ver[id];
			String card = prop.getComments();
			int cid = cardinalityToId.get(card).intValue();
			double wgt = owg.getWeight(10);//15
			addEdge(result,"card",cid,id,wgt);
			ver[id].setComments("");
		}
		return result;
	}
	
	public ArrayList<Edge> encodeEdgesBetweenClassesAndProperties(OWLOntology onto){
		ArrayList<Edge> result = new ArrayList<Edge>();
		for(int i = 0 ; i < csz ; i++){
			OWLClassImpl owlC = (OWLClassImpl)cSet[i];
			boolean auth1 = owlC != null;
			if (auth1 == true){
				URI curi = owlC.getURI();
				int cid = classToId.get(curi.toString()).intValue();
				try{
					Object[] sup = owlC.getSuperClasses().toArray();
					int supsz = sup.length;
					for (int j = 0 ; j < supsz ; j++){
						Object roc = sup[j];
						if (roc instanceof OWLObjectCardinalityRestrictionImpl){
							OWLObjectProperty op = ((OWLObjectCardinalityRestrictionImpl)roc).getObjectProperty();
							result.addAll(encodeEdgesBetweenClassAndObjectProperties(cid,op));
						}
						else{
							if (roc instanceof OWLObjectAllRestrictionImpl){
								OWLObjectProperty op = ((OWLObjectAllRestrictionImpl)roc).getObjectProperty();
								result.addAll(encodeEdgesBetweenClassAndObjectProperties(cid,op));
							}
						}
					}
				}
				catch(Exception ex){
					ex.printStackTrace();
				}
			}
		}
		return result;
	}
	
	public ArrayList<Edge> encodeEdgesBetweenClassAndObjectProperties(int cid, OWLObjectProperty op){
		ArrayList<Edge> result = new ArrayList<Edge>();
		try{
			URI opuri = op.getURI();
			int opid = relationToId.get(opuri.toString()).intValue();
			double wgt = owg.getWeight(8);
			addEdge(result,"attribution",opid,cid,wgt);
		}
		catch(Exception ex){
			ex.printStackTrace();
		}
		return result;
	}
	
	public void updatePropertyInstanceName(Vertice[] verS){
		Object[] keys = propertyInstanceToId.keySet().toArray();
		int ksz = keys.length;
		for (int i = 0 ; i < ksz ; i++){
			int vId = propertyInstanceToId.get((String)keys[i]).intValue();
			verS[vId].setLabel("PI"+Integer.toString(i));
		}
	}
	
	public OntologyGraph completeOntologyGraph(OntologyGraph og, Object[] vSet, boolean useLab){
		OntologyGraph result = og;
		currentVerticeId = result.getVerticePool().size();
		currentEdgeId = result.getEdgePool().size();
		int vsz = currentVerticeId; 
		LabelTokenizer lt = new LabelTokenizer();
		for (int i = 0; i < vsz ; i++){
			Vertice ver = (Vertice)vSet[i];
			String vCat = ver.getCategory();
			boolean generate = vCat.equalsIgnoreCase("Class") 
						|| vCat.equalsIgnoreCase("Object") 
						|| vCat.equalsIgnoreCase("Relation");
			if (generate == true){
				ArrayList<String> tokens = null;
				boolean test = ver.getCategory().equalsIgnoreCase("Object");
				int tsz = 0;
				if ((useLab == true) || (test == true)){
					String vLab = ver.getLabel();
					tokens = lt.tokenize(vLab);
					tsz = lt.getNumberOfTokens();
				}
				else{
					String vLab = ver.getComments();
					ver.setLabel(vLab);
					tokens = lt.tokenizePath(vLab);
					tsz = lt.getNumberOfTokens();
				}
				
				if (!ver.getAnnotation().equalsIgnoreCase("")){
					tokens.add(ver.getAnnotation());
					tsz++;
				}
				
				for (int j = 0 ; j < tsz ; j++){
					int tId = 0;
					if (tokenToId.containsKey(tokens.get(j))){
						tId = tokenToId.get(tokens.get(j)).intValue();
					}
					else{
						tId = directVerticeInsertion(result,"Token",tokens.get(j));
						tokenToId.put(tokens.get(j),tId);
					}
					
					double wgt = 0.0;
					if (vCat.equalsIgnoreCase("Class")){wgt = owg.getWeight(2);}
					if (vCat.equalsIgnoreCase("Object")){wgt = owg.getWeight(25);}
					if (vCat.equalsIgnoreCase("Relation")){wgt = owg.getWeight(17);}//21
					directEdgeInsertion(result,"name",tId,ver.getId(),wgt);
					//directEdgeInsertion(og,"instantiation",tId,tokId,0.0);
				}
			}
		}
		return result;
	}
	
	public int directVerticeInsertion(OntologyGraph og, String cat, String name){
		int result = currentVerticeId;
		
		Vertice v = new VerticeImpl(currentVerticeId,cat,name,"","",0.0,null,null);
		og.insertVertice(v);
		currentVerticeId++;
		
		return result;
	}
	
	public void directEdgeInsertion(OntologyGraph og, String lab, int hd, int tl, double wgt){
		Edge e1 = new EdgeImpl(currentEdgeId,lab,wgt,hd,tl);
		og.insertEdge(e1);
		((Vertice)og.getVerticePool().get(hd)).insertNeighId(tl,NAMEID,true);
		((Vertice)og.getVerticePool().get(tl)).insertNeighId(hd,NAMEID,false);
		((Vertice)og.getVerticePool().get(hd)).insertEdgeId(currentEdgeId,NAMEID,true);
		((Vertice)og.getVerticePool().get(tl)).insertEdgeId(currentEdgeId,NAMEID,false);
		currentEdgeId++;
	}
	
	public void clearAll(){
		cSet = null;
		dSet = null;
		iSet = null;
		dpSet = null;
		opSet = null;
		
		classToId.clear();
		datatypeToId.clear();
		relationToId.clear();
		propertyToId.clear();
		objectToId.clear();
		propertyInstanceToId.clear();
		dataValueToId.clear();
		nullVerticeToName.clear();
		tokenToId.clear();
		cardinalityToId.clear();
	}

}
