/**
 * 
 */
package ca.uqam.info.latece.sboa.impl.ograph;

import java.util.Vector;
import java.util.Hashtable;
import java.util.ArrayList;

import ca.uqam.info.latece.sboa.inter.ograph.Edge;
import ca.uqam.info.latece.sboa.inter.ograph.EdgesPool;

/**
 * @author J.F. Djoufak
 *
 */
public class EdgesPoolImpl implements EdgesPool {

	private String[] labelNames = null;
	
	private Hashtable<String,Integer> edgeLabelToId = new Hashtable<String,Integer>();
	private Vector<ArrayList<Integer>> idsGroupedByLabel = null;
	private Hashtable<Integer,Edge> edgeIdToEdges = new Hashtable<Integer,Edge>();
	
	private int[] labelGroupsLen = null;
	private int size = 0;
	private int numberOfLabels = 0;
	
	public EdgesPoolImpl(){
		
	}
	
	public EdgesPoolImpl(String[] cat){
		init(cat);
	}
	
	public int numberOfPart() {
		return numberOfLabels;
	}
	
	public int getPartId(String label){
		return edgeLabelToId.get(label).intValue();
	}

	public boolean contains(String lab) {
		return edgeLabelToId.containsKey(lab);
	}

	public boolean contains(int Id) {
		Integer I = new Integer(Id);
		return edgeIdToEdges.containsKey(I);
	}

	public Object get(int Id) {
		Integer I = new Integer(Id);
		return edgeIdToEdges.get(I);
	}

	public Object[] getAll(String lab) {
		Integer lId = edgeLabelToId.get(lab);
		int labLen = labelGroupsLen[lId.intValue()];
		Object[] result = new Object[labLen];
		ArrayList<Integer> idOfLab = (ArrayList<Integer>)idsGroupedByLabel.elementAt(lId.intValue());
		for(int i=0 ; i < labLen ; i++){
			Integer I = (Integer)idOfLab.get(i);
			result[i] = get(I.intValue());
		}
		return result;
	}
	
	public Object[] getAll(){
		ArrayList<Object> vSet = new ArrayList<Object>();
		for (int i = 0 ; i < numberOfLabels ; i++){
			Object[] data = getAll(labelNames[i]);
			int dSz = this.partSize(labelNames[i]);
			for (int j = 0 ; j < dSz ; j++){
				vSet.add(data[j]);
			}
		}
		return vSet.toArray();
	}
	
	public Object[] getEdgesBetween(ArrayList<Integer> s1, ArrayList<Integer> s2){
		ArrayList<Integer> edgIds = new ArrayList<Integer>();
		int size = 0;
		for (int i = 0 ; i < numberOfLabels ; i++){
			for (int j = 0 ; j < labelGroupsLen[i] ; j++){
				int edgId = idsGroupedByLabel.elementAt(i).get(j);
				Integer I = new Integer(edgId);
				Edge edg = (Edge)edgeIdToEdges.get(I);
				Integer Hd = new Integer(edg.getHead());
				Integer Tl = new Integer(edg.getTail());
				if ( (s1.contains(Hd)) && (s2.contains(Tl)) ){
					edgIds.add(I);
					size++;
				}
			}
		}
		Object[] result = new Object[size];
		for (int j = 0 ; j < size ; j++){
			Integer I = (Integer)edgIds.get(j);
			result[j] = get(I.intValue());
		}
		return result;
	}

	public Object[] getAll(int[] Id) {
		int nbIds = Id.length;
		Object[] result = new Object[nbIds];
		for(int i =0 ; i < nbIds ; i++){
			result[i] = get(Id[i]);
		}
		return result;
	}
	
	public String[] getPartsNames(){
		return labelNames;
	}

	public void init(String[] lab) {
		int lsize = lab.length;
		labelNames = new String[lsize];
		idsGroupedByLabel = new Vector<ArrayList<Integer>>(lsize);
		labelGroupsLen = new int[lsize];
		for(int i=0 ; i<lsize ; i++) {
			labelNames[i] = lab[i];
			edgeLabelToId.put(labelNames[i],i);
			idsGroupedByLabel.insertElementAt(new ArrayList<Integer>(),i);
			labelGroupsLen[i] = 0;
			numberOfLabels++;
		}
	}

	public void insert(Object obj) {
		Edge edg = (Edge) obj;
		String elab = edg.getLabel();
		int eid = edg.getId();
		Integer Id = new Integer(eid);
		Integer labId = (Integer)edgeLabelToId.get(elab);
		idsGroupedByLabel.elementAt(labId.intValue()).add(Id);
		edgeIdToEdges.put(Id,edg);
		labelGroupsLen[labId]++;
		size++;
	}

	public void insertAll(Object[] objSet) {
		int oSize = objSet.length;
		for(int i = 0 ; i < oSize ; i++){
			insert(objSet[i]);
		}
	}

	public boolean isEmpty() {
		return (size == 0);
	}
	
	public int partSize(String name){
		Integer Id = (Integer)edgeLabelToId.get(name);
		return labelGroupsLen[Id.intValue()];
	}

	public void remove(int Id) {
		Edge edg = (Edge)get(Id);
		String lab = edg.getLabel();
		Integer labId = (Integer)edgeLabelToId.get(lab);
		Integer I = new Integer(Id);
		idsGroupedByLabel.elementAt(labId).remove(I);
		labelGroupsLen[labId.intValue()]--;
		size--;
		edgeIdToEdges.remove(I);
		edgeLabelToId.remove(lab);
	}
	
	public void removeAll(String lab) {
		Integer labId = (Integer)edgeLabelToId.get(lab);
		int cSize = labelGroupsLen[labId.intValue()];
		ArrayList<Integer> labList = (ArrayList<Integer>)idsGroupedByLabel.elementAt(labId.intValue());
		for (int i = 0 ; i < cSize ; i++){
			Integer I = (Integer)labList.get(i);
			remove(I.intValue());
		}
	}

	public void removeAll(int[] Id) {
		int idSize = Id.length;
		for (int i = 0 ; i < idSize ; i++){
			remove(Id[i]);
		}
	}

	public int size() {
		return size;
	}
	
	public void viewAllEdges(){
		for (int i = 0 ; i < numberOfLabels ; i++){
			String name = labelNames[i];
			Object[] verVect = getAll(name);
			int sz = partSize(name);
			for (int j = 0 ;j < sz ; j++){
				((Edge)verVect[j]).view();
			}
		}
	}
	
	public void clearAll(){
		labelNames = null;
		edgeLabelToId.clear();
		idsGroupedByLabel.clear();
		edgeIdToEdges.clear();
	}

}
