/**
 * 
 */
package ca.uqam.info.latece.sboa.impl.ograph;

import java.util.ArrayList;
import java.util.Vector;
import java.util.TreeSet;

import org.semanticweb.owl.model.OWLEntity;

import ca.uqam.info.latece.sboa.inter.ograph.Vertice;

/**
 * @author J.F. Djoufak
 *
 */
public class VerticeImpl implements Vertice {
	
	private int Id;
	private String category;
	private String label;
	private String comments;
	private String annotation;
	private double score;
	private OWLEntity oe1;
	private OWLEntity oe2;
	
	private Vector<TreeSet<Integer>> tailNeighborhood = null;
	private Vector<TreeSet<Integer>> headNeighborhood = null;
	
	private Vector<ArrayList<Integer>> tailOfEdges = null;
	private Vector<ArrayList<Integer>> headOfEdges = null;
	private int[] tNeighLen = null;
	private int[] hNeighLen = null;
	private int tNeighSz = 0;
	private int hNeighSz = 0;
	private int nOfEdgeslab = 0;
	
	public VerticeImpl(){
		
	}
	
	public VerticeImpl(int id, String cat, String lab, String comm, String ann, double wgt, OWLEntity e1,  OWLEntity e2){
		setCategory(cat);
		setId(id);
		setLabel(lab);
		setComments(comm);
		setWeight(wgt);
		setFirstEntity(e1);
		setSecondEntity(e2);
		setAnnotation(ann);
	}

	public String getCategory() {
		return category;
	}
	
	public String getComments(){
		return comments;
	}
	
	public String getAnnotation(){
		return annotation;
	}
	public OWLEntity getFirstEntity(){
		return oe1;
	}
	
	public OWLEntity getSecondEntity(){
		return oe2;
	}
	
	public int getId(){
		return Id;
	}

	public String getLabel() {
		return label;
	}

	public double getWeight() {
		return score;
	}

	public void setCategory(String cat) {
		category = cat;
	}
	
	public void setComments(String comm){
		comments = comm;
	}
	
	public void setAnnotation(String ann){
		annotation = ann;
	}
	
	public void setFirstEntity(OWLEntity e1){
		oe1 = e1;
	}
	
	public void setSecondEntity(OWLEntity e2){
		oe2 = e2;
	}
	
	public void setId(int i){
		Id = i;
	}

	public void setLabel(String lab) {
		label = lab;
	}

	public void setWeight(double wgt) {
		score = wgt;
	}
	
	public void view(){
		System.out.println("Vertice-"+Id);
		System.out.println("	Category = "+category);
		System.out.println("	Label = "+label);
		System.out.println("	Score = "+score);
		System.out.println("	Comments = "+comments);
		System.out.println("	Annotation = "+annotation);
		System.out.println("	Head Neighbor of "+hNeighSz+" vertice : "+getNeighborhood(true).toString());
		System.out.println("	Tail Neighbor of "+tNeighSz+" vertice : "+getNeighborhood(false).toString());
	}
	
	public void clearAll(){
		oe1 = null;
		oe2 = null;
		label = null;
		category = null;
		comments = null;
		tailNeighborhood.clear();
		tailNeighborhood = null;
		headNeighborhood.clear();
		headNeighborhood = null;
		tailOfEdges.clear();
		tailOfEdges = null;
		headOfEdges.clear();
		headOfEdges = null;
		tNeighLen = null;
		hNeighLen = null;
	}
	
	public void setNumberOfEdgesLabels(int number){
		nOfEdgeslab = number;
		tNeighSz = 0;
		hNeighSz = 0;
		tNeighLen = new int[nOfEdgeslab];
		hNeighLen = new int[nOfEdgeslab];
		tailNeighborhood = new Vector<TreeSet<Integer>>(nOfEdgeslab);
		headNeighborhood = new Vector<TreeSet<Integer>>(nOfEdgeslab);
		tailOfEdges = new Vector<ArrayList<Integer>>(nOfEdgeslab);
		headOfEdges = new Vector<ArrayList<Integer>>(nOfEdgeslab);
		for (int i = 0 ; i < nOfEdgeslab ; i++){
			tailNeighborhood.insertElementAt(new TreeSet<Integer>(),i);
			headNeighborhood.insertElementAt(new TreeSet<Integer>(),i);
			tailOfEdges.insertElementAt(new ArrayList<Integer>(),i);
			headOfEdges.insertElementAt(new ArrayList<Integer>(),i);
			tNeighLen[i] = 0;
			hNeighLen[i] = 0;
		}
	}
	
	public void insertNeighId(int vId, int labId, boolean isHead){
		Integer I = new Integer(vId);
		if (isHead == true){
			boolean insT = headNeighborhood.elementAt(labId).add(I);
			if (insT == true){
				hNeighLen[labId]++;
				hNeighSz++;
			}
		}
		else{
			boolean insT = tailNeighborhood.elementAt(labId).add(I);
			if (insT == true){
				tNeighLen[labId]++;
				tNeighSz++;
			}
		}
	}
	
	public void insertEdgeId(int eId, int labId, boolean isHead){
		Integer I = new Integer(eId);
		if (isHead == true){
			headOfEdges.elementAt(labId).add(I);
		}
		else{
			tailOfEdges.elementAt(labId).add(I);
		}
	}
	
	public ArrayList<Integer> getNeighborhood(int labId, boolean isHead){
		if (isHead == true){
			TreeSet<Integer> tree = headNeighborhood.elementAt(labId);
			return new ArrayList<Integer>(tree);
		}
		else{
			TreeSet<Integer> tree = tailNeighborhood.elementAt(labId);
			return new ArrayList<Integer>(tree);
		}
	}
	
	public ArrayList<Integer> getEdges(int labId, boolean isHead){
		if (isHead == true){
			return headOfEdges.elementAt(labId);
		}
		else{
			return tailOfEdges.elementAt(labId);
		}
	}
	
	public ArrayList<Integer> getNeighborhood(boolean isHead){
		ArrayList<Integer> result = new ArrayList<Integer>();
		if (isHead == true){
			for (int i = 0 ; i < nOfEdgeslab ; i++){
				ArrayList<Integer> r = getNeighborhood(i,true);
				result.addAll(r);
			}
		}
		else{
			for (int j = 0 ; j < nOfEdgeslab ; j++){
				ArrayList<Integer> r = getNeighborhood(j,false);
				result.addAll(r);
			}
		}
		return result;
	}
	
	public ArrayList<Integer> getEdges(boolean isHead){
		ArrayList<Integer> result = new ArrayList<Integer>();
		if (isHead == true){
			for (int i = 0 ; i < nOfEdgeslab ; i++){
				ArrayList<Integer> r = getEdges(i,true);
				result.addAll(r);
			}
		}
		else{
			for (int j = 0 ; j < nOfEdgeslab ; j++){
				ArrayList<Integer> r = getEdges(j,false);
				result.addAll(r);
			}
		}
		return result;
	}
	
	public int getNeighborhoodSize(int labId, boolean isHead){
		if (isHead == true){
			return hNeighLen[labId];
		}
		else{
			return tNeighLen[labId];
		}
	}
	
	public int getNeighborhoodSize(boolean isHead){
		if (isHead == true){
			return hNeighSz;
		}
		else{
			return tNeighSz;
		}
	}

}
