/**
 * 
 */
package ca.uqam.info.latece.sboa.impl.selection;

import java.net.URI;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.HashSet;

import no.uib.cipr.matrix.Matrix;
import no.uib.cipr.matrix.DenseMatrix;

import org.semanticweb.owl.model.OWLEntity;

import ca.uqam.info.latece.sboa.inter.selection.AlignPairs;

import ca.uqam.info.latece.sboa.inter.selection.AssignmentMethods;
import ca.uqam.info.latece.sboa.impl.selection.AssignmentMethodsImpl;

/**
 * @author J.F. Djoufak
 *
 */
public class AlignPairsImpl implements AlignPairs {

	private ArrayList<ArrayList<Object>> pool = new ArrayList<ArrayList<Object>>();
	
	private Hashtable<URI,HashSet<Integer>> LUris = new Hashtable<URI,HashSet<Integer>>();
	private Hashtable<URI,HashSet<Integer>> RUris = new Hashtable<URI,HashSet<Integer>>();
	
	private Hashtable<URI,Integer> LUPos = new Hashtable<URI,Integer>();
	private Hashtable<URI,Integer> RUPos = new Hashtable<URI,Integer>();
	
	private Hashtable<Double,HashSet<Integer>> Sim = new Hashtable<Double,HashSet<Integer>>();
	private Matrix refIds = null;
	
	private int poolSz = 0;

	public AlignPairsImpl() {
		
	}

	public void addPair(OWLEntity ent1, OWLEntity ent2, double score) {
		Double Sc = new Double(score);
		ArrayList<Object> triple = new ArrayList<Object>();
		triple.add(ent1);triple.add(ent2);triple.add(Sc);
		pool.add(triple);
		try{
			URI luri = ent1.getURI();
			URI ruri = ent2.getURI();
			LUris = insertURI(LUris,luri,poolSz);
			RUris = insertURI(RUris,ruri,poolSz);
			Sim = insertSim(Sim,new Double(score),poolSz);
		}
		catch(Exception e){
			e.printStackTrace();
		}
		poolSz++;
	}
	
	private Hashtable<URI,HashSet<Integer>> insertURI(Hashtable<URI,HashSet<Integer>> ht, URI uri, int pos){
		if (!ht.containsKey(uri)){
			HashSet<Integer> list = new HashSet<Integer>();
			list.add(new Integer(pos));
			ht.put(uri,list);
		}
		else{
			HashSet<Integer> list = ht.get(uri);
			list.add(new Integer(pos));
			ht.remove(uri);
			ht.put(uri,list);
		}
		return ht;
	}
	
	private Hashtable<Double,HashSet<Integer>> insertSim(Hashtable<Double,HashSet<Integer>> ht, Double sim, int pos){
		if (!ht.containsKey(sim)){
			HashSet<Integer> list = new HashSet<Integer>();
			list.add(new Integer(pos));
			ht.put(sim,list);
		}
		else{
			HashSet<Integer> list = ht.get(sim);
			list.add(new Integer(pos));
			ht.remove(sim);
			ht.put(sim,list);
		}
		return ht;
	}
	
	private Hashtable<ArrayList<OWLEntity>, Double> insertCouple(Hashtable<ArrayList<OWLEntity>, Double> ht,
			HashSet<Integer> tripleId, boolean avoidUnit){
		Object[] Ids = tripleId.toArray();
		int IdSz = Ids.length;
		for (int i = 0 ; i < IdSz ; i++){
			int id = ((Integer)Ids[i]).intValue();
			ArrayList triple = pool.get(id);
			double val = ((Double)triple.get(2)).doubleValue();
			ArrayList<OWLEntity> couple = new ArrayList<OWLEntity>();
			couple.add((OWLEntity)triple.get(0));
			couple.add((OWLEntity)triple.get(1));
			if (ht.contains(couple)){
				double prevVal = ht.get(couple).doubleValue();
				if (prevVal < val){
					ht.remove(couple);
					ht.put(couple,new Double(val));
				}
			}
			else{
				if ((!avoidUnit) || ((avoidUnit) && (val != 1.0))){
					ht.put(couple,new Double(val));
				}
			}
		}
		return ht;
	}
	
	public Hashtable<ArrayList<OWLEntity>, Double> thresholdSelect(double threshold, boolean avoidUnit) {
		Hashtable<ArrayList<OWLEntity>, Double> result = new Hashtable<ArrayList<OWLEntity>, Double>();
		Object[] keys = Sim.keySet().toArray();
		int ksz = keys.length;
		for (int i = 0 ; i < ksz ; i++){
			Double kval = (Double)keys[i];
			if (kval.doubleValue() >= threshold){
				HashSet<Integer> tripleIds = Sim.get(keys[i]);
				result = insertCouple(result,tripleIds,avoidUnit);
			}
		}
		return result;
	}
	
	public Hashtable<ArrayList<OWLEntity>, Double> maxSelect(boolean avoidUnit, boolean all) {
		Hashtable<ArrayList<OWLEntity>, Double> result = new Hashtable<ArrayList<OWLEntity>, Double>();
		HashSet<Integer> resIds = new HashSet<Integer>();
		resIds.addAll(getMaxScores(LUris,all));
		resIds.addAll(getMaxScores(RUris,all));
		result = insertCouple(result,resIds,avoidUnit);
		return result;
	}
	
	private HashSet<Integer> getMaxScores(Hashtable<URI,HashSet<Integer>> ht, boolean all){
		HashSet<Integer> result = new HashSet<Integer>();
		double[] max = getMaxSim(ht);
		Object[] keys = ht.keySet().toArray();
		int size = keys.length;
		for (int i = 0 ; i < size ; i++){
			Object[] posSet = ht.get(keys[i]).toArray();
			int pSz = posSet.length;
			boolean choosed = false;
			for (int j = 0 ; j < pSz ; j++){
				int pos = ((Integer)posSet[j]).intValue();
				ArrayList pLine = pool.get(pos);
				double sim = ((Double)pLine.get(2)).doubleValue();
				if ((sim >= max[i]) && (choosed == false)){
					result.add(new Integer(pos));
					if (all == false){choosed = true;}
				}
			}
		}
		return result;
	}
	
	private double[] getMaxSim(Hashtable<URI,HashSet<Integer>> ht){
		Object[] keys = ht.keySet().toArray();
		int size = keys.length;
		double[] result = new double[size];
		for (int i = 0 ; i < size ; i++){result[i] = 0;}
		for (int i = 0 ; i < size ; i++){
			Object[] posSet = ht.get(keys[i]).toArray();
			int pSz = posSet.length;
			for (int j = 0 ; j < pSz ; j++){
				int pos = ((Integer)posSet[j]).intValue();
				ArrayList pLine = pool.get(pos);
				double sim = ((Double)pLine.get(2)).doubleValue();
				if (sim > result[i]){result[i] = sim;}
			}
		}
		return result;
	}
	
	public Hashtable<ArrayList<OWLEntity>, Double> hungarianSelect(boolean avoidUnit){
		Hashtable<ArrayList<OWLEntity>,Double> result = new Hashtable<ArrayList<OWLEntity>,Double>();
		HashSet<Integer> resIds = new HashSet<Integer>();
		int nbRC = Math.max(LUris.size(),RUris.size());
		refIds = new DenseMatrix(nbRC,nbRC);
		Matrix sim = new DenseMatrix(nbRC,nbRC);
		sim = buildSelectionMatrix(sim,LUris,RUris,LUPos,RUPos);
		System.out.println("Matrix builded ["+sim.numRows()+","+sim.numColumns()+"]");
		AssignmentMethods am = new AssignmentMethodsImpl();
		Matrix selection = am.hungarian(sim);
		System.out.println("Selection done ["+selection.numRows()+","+selection.numColumns()+"]");
		int sz = selection.numRows();
		for (int j = 0 ; j < sz ; j++){
			int lineId = (int)selection.get(j,0);
			int colId = (int)selection.get(j,1);
			Integer ref = new Integer((int)refIds.get(lineId,colId));
			resIds.add(ref);
		}
		result = insertCouple(result,resIds,avoidUnit);
		return result;
	}
	
	private Matrix buildSelectionMatrix(Matrix matrix, Hashtable<URI,HashSet<Integer>> ht1, Hashtable<URI,HashSet<Integer>> ht2,
			Hashtable<URI,Integer> L1, Hashtable<URI,Integer> L2){
		Object[] keys1 = ht1.keySet().toArray();
		Object[] keys2 = ht2.keySet().toArray();
		int sz1 = ht1.size();int sz2 = ht2.size();
		for (int i = 0 ; i < sz1 ; i++){L1.put((URI)keys1[i],i);}
		for (int i = 0 ; i < sz2 ; i++){L2.put((URI)keys2[i],i);}
		for (int i = 0 ; i < sz1 ; i++){
			URI uri1 = (URI)keys1[i];
			int indI = ((Integer)L1.get(uri1)).intValue();
			Object[] posSet = ht1.get(keys1[i]).toArray();
			int pSz = posSet.length;
			for (int j = 0 ; j < pSz ; j++){
				int pos = ((Integer)posSet[j]).intValue();
				ArrayList pLine = pool.get(pos);
				double sim = ((Double)pLine.get(2)).doubleValue();
				try{
					URI uri2 = ((OWLEntity)pLine.get(1)).getURI();
					int indJ = ((Integer)L2.get(uri2)).intValue();
					double cvalue = matrix.get(indI,indJ);
					if (cvalue < sim){
						matrix.set(indI,indJ,sim);
						refIds.set(indI,indJ,pos);
					}
				}
				catch(Exception ex){
					ex.printStackTrace();
				}
				
			}
		}
		return matrix;
	}
}
